@@ -114,7 +114,7 @@ static pa_sink* find_evacuation_sink(pa_core *c, pa_sink_input *i, pa_sink *skip
         if (target == skip)
             continue;
 
-        if (!PA_SINK_IS_LINKED(pa_sink_get_state(target)))
+        if (!PA_SINK_IS_LINKED(target->state))
             continue;
 
         if (!pa_sink_input_may_move_to(i, target))
@@ -136,7 +136,7 @@ static pa_sink* find_evacuation_sink(pa_core *c, pa_sink_input *i, pa_sink *skip
         target = fb_sink;
 
     if (!target)
-	pa_log_debug("No evacuation sink found.");
+        pa_log_debug("No evacuation sink found.");
 
     return target;
 }
@@ -224,7 +224,7 @@ static pa_source* find_evacuation_source(pa_core *c, pa_source_output *o, pa_sou
         if (skip && !target->monitor_of != !skip->monitor_of)
             continue;
 
-        if (!PA_SOURCE_IS_LINKED(pa_source_get_state(target)))
+        if (!PA_SOURCE_IS_LINKED(target->state))
             continue;
 
         if (!pa_source_output_may_move_to(o, target))
