@@ -41,8 +41,6 @@
 #include <pulsecore/rtpoll.h>
 #include <pulsecore/proplist-util.h>
 
-#include "module-tunnel-source-new-symdef.h"
-
 PA_MODULE_AUTHOR("Alexander Couzens");
 PA_MODULE_DESCRIPTION("Create a network source which connects via a stream to a remote PulseAudio server");
 PA_MODULE_VERSION(PACKAGE_VERSION);
@@ -430,30 +428,44 @@ static int source_process_msg_cb(pa_msgobject *o, int code, void *data, int64_t
 
             return 0;
         }
-        case PA_SOURCE_MESSAGE_SET_STATE:
-            if (!u->stream || pa_stream_get_state(u->stream) != PA_STREAM_READY)
-                break;
-
-            switch ((pa_source_state_t) PA_PTR_TO_UINT(data)) {
-                case PA_SOURCE_SUSPENDED: {
-                    cork_stream(u, true);
-                    break;
-                }
-                case PA_SOURCE_IDLE:
-                case PA_SOURCE_RUNNING: {
-                    cork_stream(u, false);
-                    break;
-                }
-                case PA_SOURCE_INVALID_STATE:
-                case PA_SOURCE_INIT:
-                case PA_SOURCE_UNLINKED:
-                    break;
-            }
-            break;
     }
     return pa_source_process_msg(o, code, data, offset, chunk);
 }
 
+/* Called from the IO thread. */
+static int source_set_state_in_io_thread_cb(pa_source *s, pa_source_state_t new_state, pa_suspend_cause_t new_suspend_cause) {
+    struct userdata *u;
+
+    pa_assert(s);
+    pa_assert_se(u = s->userdata);
+
+    /* It may be that only the suspend cause is changing, in which case there's
+     * nothing to do. */
+    if (new_state == s->thread_info.state)
+        return 0;
+
+    if (!u->stream || pa_stream_get_state(u->stream) != PA_STREAM_READY)
+        return 0;
+
+    switch (new_state) {
+        case PA_SOURCE_SUSPENDED: {
+            cork_stream(u, true);
+            break;
+        }
+        case PA_SOURCE_IDLE:
+        case PA_SOURCE_RUNNING: {
+            cork_stream(u, false);
+            break;
+        }
+        case PA_SOURCE_INVALID_STATE:
+        case PA_SOURCE_INIT:
+        case PA_SOURCE_UNLINKED:
+            break;
+    }
+
+    return 0;
+}
+
 int pa__init(pa_module *m) {
     struct userdata *u = NULL;
     pa_modargs *ma = NULL;
@@ -543,6 +555,7 @@ int pa__init(pa_module *m) {
     pa_source_new_data_done(&source_data);
     u->source->userdata = u;
     u->source->parent.process_msg = source_process_msg_cb;
+    u->source->set_state_in_io_thread = source_set_state_in_io_thread_cb;
     u->source->update_requested_latency = source_update_requested_latency_cb;
 
     pa_source_set_asyncmsgq(u->source, u->thread_mq->inq);
