@@ -53,8 +53,6 @@
 #include <pulsecore/poll.h>
 #include <pulsecore/arpa-inet.h>
 
-#include "module-rtp-recv-symdef.h"
-
 #include "rtp.h"
 #include "sdp.h"
 #include "sap.h"
@@ -527,7 +525,7 @@ static struct session *session_new(struct userdata *u, const pa_sdp_info *sdp_in
         goto fail;
 
     pa_sink_input_new_data_init(&data);
-    pa_sink_input_new_data_set_sink(&data, sink, false);
+    pa_sink_input_new_data_set_sink(&data, sink, false, true);
     data.driver = __FILE__;
     pa_proplist_sets(data.proplist, PA_PROP_MEDIA_ROLE, "stream");
     pa_proplist_setf(data.proplist, PA_PROP_MEDIA_NAME,
