@@ -66,6 +66,7 @@ PA_MODULE_USAGE(
         "loop=<loopback to local host?> "
         "ttl=<ttl value> "
         "inhibit_auto_suspend=<always|never|only_with_non_monitor_sources>"
+        "stream_name=<name of the stream>"
 );
 
 #define DEFAULT_PORT 46000
@@ -90,6 +91,7 @@ static const char* const valid_modargs[] = {
     "loop",
     "ttl",
     "inhibit_auto_suspend",
+    "stream_name",
     NULL
 };
 
@@ -469,7 +471,9 @@ int pa__init(pa_module*m) {
     k = sizeof(sa_dst);
     pa_assert_se((r = getsockname(fd, (struct sockaddr*) &sa_dst, &k)) >= 0);
 
-    n = pa_sprintf_malloc("PulseAudio RTP Stream on %s", pa_get_fqdn(hn, sizeof(hn)));
+    n = pa_xstrdup(pa_modargs_get_value(ma, "stream_name", NULL));
+    if (n == NULL)
+        n = pa_sprintf_malloc("PulseAudio RTP Stream on %s", pa_get_fqdn(hn, sizeof(hn)));
 
     if (af == AF_INET) {
         p = pa_sdp_build(af,
@@ -514,11 +518,6 @@ fail:
     if (sap_fd >= 0)
         pa_close(sap_fd);
 
-    if (o) {
-        pa_source_output_unlink(o);
-        pa_source_output_unref(o);
-    }
-
     return -1;
 }
 
