@@ -54,6 +54,10 @@ static pa_strlist *load_makefile() {
 }
 
 START_TEST (mixer_path_test) {
+#ifdef MESON_BUILD
+    pa_log_info("Test disabled for meson build");
+    return;
+#else
     DIR *dir;
     struct dirent *ent;
     pa_strlist *ship = load_makefile();
@@ -83,6 +87,7 @@ START_TEST (mixer_path_test) {
     }
     closedir(dir);
     pa_strlist_free(ship);
+#endif
 }
 END_TEST
 
